@@ -20,14 +20,15 @@
  * Author: David Lutterkort <dlutter@redhat.com>
  */
 
+#include <config.h>
 #include "augeas.h"
 #include "internal.h"
 #include "memory.h"
-#include "config.h"
 #include "syntax.h"
 
 #include <fnmatch.h>
 #include <argz.h>
+#include <string.h>
 
 /* We always have a toplevel entry for P_ROOT */
 #define P_ROOT   "augeas"
@@ -135,7 +136,9 @@ static struct path *make_path(const struct tree *root, const char *path) {
     for (const char *p = path; *p != '\0'; p++) {
         if (*p == SEP) {
             while (*p == SEP) p++;
-            if (*p) result->nsegments++;
+            if (*p == '\0')
+                break;
+            result->nsegments++;
         }
     }
     if (result->nsegments == 0)
@@ -584,7 +587,7 @@ struct tree *tree_set(struct tree *root, const char *path, const char *value) {
     free_path(p);
 
     if (tree->value != NULL) {
-        free((char *) tree->value);
+        free(tree->value);
         tree->value = NULL;
     }
     if (value != NULL) {
@@ -603,15 +606,15 @@ int aug_set(struct augeas *aug, const char *path, const char *value) {
     return tree_set(aug->tree, path, value) == NULL ? -1 : 0;
 }
 
-int aug_insert(struct augeas *aug, const char *path, const char *label,
-               int before) {
+int tree_insert(struct tree **tree, const char *path, const char *label,
+                int before) {
     struct path *p = NULL;
     struct tree *new = NULL;
 
     if (strchr(label, SEP) != NULL)
         return -1;
 
-    p = make_path(aug->tree, path);
+    p = make_path(*tree, path);
     if (p == NULL)
         goto error;
 
@@ -624,11 +627,11 @@ int aug_insert(struct augeas *aug, const char *path, const char *label,
 
     struct segment *seg = last_segment(p);
     if (before) {
-        struct tree *siblings = seg_siblings(p, seg);
-        if (siblings == aug->tree) {
-            list_insert_before(new, seg->tree, aug->tree);
+        if (seg == p->segments) {
+            list_insert_before(new, seg->tree, *tree);
         } else {
-            list_insert_before(new, seg->tree, siblings);
+            struct tree *parent = seg_parent(p, seg);
+            list_insert_before(new, seg->tree, parent->children);
         }
     } else {
         new->next = seg->tree->next;
@@ -642,8 +645,12 @@ int aug_insert(struct augeas *aug, const char *path, const char *label,
     return -1;
 }
 
-struct tree *make_tree(const char *label, const char *value,
-                       struct tree *children) {
+int aug_insert(struct augeas *aug, const char *path, const char *label,
+               int before) {
+    return tree_insert(&(aug->tree), path, label, before);
+}
+
+struct tree *make_tree(char *label, char *value, struct tree *children) {
     struct tree *tree;
     CALLOC(tree, 1);
     if (tree == NULL)
@@ -661,8 +668,8 @@ static void free_tree_node(struct tree *tree) {
     if (tree == NULL)
         return;
 
-    free((char *) tree->label);
-    free((char *) tree->value);
+    free(tree->label);
+    free(tree->value);
     free(tree);
 }
 
