@@ -20,9 +20,9 @@
  * Author: David Lutterkort <dlutter@redhat.com>
  */
 
+#include <config.h>
 #include "augeas.h"
 #include "internal.h"
-#include "config.h"
 
 #include <readline/readline.h>
 #include <readline/history.h>
@@ -37,7 +37,7 @@ struct command {
     const char *help;
 };
 
-static struct command commands[];
+static const struct command const commands[];
 
 static augeas *aug = NULL;
 static const char *const progname = "augtool";
@@ -46,6 +46,8 @@ const char *root = NULL;
 char *loadpath = NULL;
 
 static char *cleanpath(char *path) {
+    if (strlen(path) == 0)
+        return path;
     char *e = path + strlen(path) - 1;
     while (e != path && (*e == SEP || isspace(*e)))
         *e-- = '\0';
@@ -187,9 +189,8 @@ static void cmd_get(char *args[]) {
     printf("%s", path);
     if (aug_get(aug, path, &val) != 1) {
         printf(" (o)\n");
-        return;
     } else if (val == NULL) {
-        val = "(none)";
+        printf(" (none)\n");
     } else {
         printf(" = %s\n", val);
     }
@@ -229,7 +230,7 @@ static void cmd_ins(char *args[]) {
 }
 
 static void cmd_help(ATTRIBUTE_UNUSED char *args[]) {
-    struct command *c;
+    const struct command *c;
 
     printf("Commands:\n\n");
     printf("    exit, quit\n        Exit the program\n\n");
@@ -297,7 +298,7 @@ static char *parseline(char *line, int maxargs, char *args[]) {
     return cmd;
 }
 
-static struct command commands[] = {
+static const struct command const commands[] = {
     { "ls",  1, 1, cmd_ls, "ls <PATH>",
       "List the direct children of PATH"
     },
@@ -341,7 +342,7 @@ static struct command commands[] = {
 
 static int run_command(char *cmd, int maxargs, char **args) {
     int r = 0;
-    struct command *c;
+    const struct command *c;
 
     if (STREQ("exit", cmd) || STREQ("quit", cmd)) {
         exit(EXIT_SUCCESS);
