@@ -20,6 +20,8 @@
  * Author: David Lutterkort <dlutter@redhat.com>
  */
 
+#include <config.h>
+
 #include <ctype.h>
 #include <stdarg.h>
 
@@ -122,23 +124,20 @@ fread_file_lim (FILE *stream, size_t max_len, size_t *length)
 
 char* read_file(const char *path) {
     FILE *fp = fopen(path, "r");
-    char *result = NULL;
+    char *result;
     size_t len;
 
     if (!fp)
         return NULL;
 
     result = fread_file_lim(fp, MAX_READ_LEN, &len);
-    if (result == NULL)
-        goto error;
+    fclose (fp);
 
-    if (len > MAX_READ_LEN || (int) len != len)
-        goto error;
+    if (result != NULL
+        && len <= MAX_READ_LEN
+        && (int) len == len)
+        return result;
 
-    return result;
- error:
-    if (fp)
-        fclose(fp);
     free(result);
     return NULL;
 }
@@ -155,7 +154,7 @@ char *unescape(const char *s, int len) {
     char *result, *t;
     int i;
 
-    if (len > strlen(s))
+    if (len < 0 || len > strlen(s))
         len = strlen(s);
 
     size = 0;
@@ -185,7 +184,7 @@ char *escape(const char *text, int cnt) {
         cnt = strlen(text);
 
     for (int i=0; i < cnt; i++) {
-        if (strchr(escape_chars, text[i]) != NULL)
+        if (text[i] && (strchr(escape_chars, text[i]) != NULL))
             len += 2;  /* Escaped as '\x' */
         else if (! isprint(text[i]))
             len += 4;  /* Escaped as '\ooo' */
@@ -196,11 +195,12 @@ char *escape(const char *text, int cnt) {
     e = esc;
     for (int i=0; i < cnt; i++) {
         char *p;
-        if ((p = strchr(escape_chars, text[i])) != NULL) {
+        if (text[i] && ((p = strchr(escape_chars, text[i])) != NULL)) {
             *e++ = '\\';
             *e++ = escape_names[p - escape_chars];
         } else if (! isprint(text[i])) {
             sprintf(e, "\\%03o", text[i]);
+            e += 4;
         } else {
             *e++ = text[i];
         }
@@ -229,27 +229,77 @@ int print_chars(FILE *out, const char *text, int cnt) {
 }
 
 char *format_pos(const char *text, int pos) {
-    char *buf;
-    size_t size;
-    FILE *stream = open_memstream(&buf, &size);
-    print_pos(stream, text, pos);
-    fclose(stream);
+    static const int window = 28;
+    char *buf = NULL, *left = NULL, *right = NULL;
+    int before = pos;
+    int llen, rlen;
+    int r;
+
+    if (before > window)
+        before = window;
+    left = escape(text + pos - before, before);
+    if (left == NULL)
+        goto done;
+    right = escape(text + pos, window);
+    if (right == NULL)
+        goto done;
+
+    llen = strlen(left);
+    rlen = strlen(right);
+    if (llen < window && rlen < window) {
+        r = asprintf(&buf, "%*s%s|=|%s%-*s\n", window - llen, "<", left,
+                     right, window - rlen, ">");
+    } else if (strlen(left) < window) {
+        r = asprintf(&buf, "%*s%s|=|%s>\n", window - llen, "<", left, right);
+    } else if (strlen(right) < window) {
+        r = asprintf(&buf, "<%s|=|%s%-*s\n", left, right, window - rlen, ">");
+    } else {
+        r = asprintf(&buf, "<%s|=|%s>\n", left, right);
+    }
+    if (r < 0) {
+        buf = NULL;
+    }
+
+ done:
+    free(left);
+    free(right);
     return buf;
 }
 
 void print_pos(FILE *out, const char *text, int pos) {
-    static const int window = 28;
-    int before = pos;
-    int total;
-    if (before > window)
-        before = window;
-    total = print_chars(NULL, text + pos - before, before);
-    if (total < window)
-        fprintf(out, "%*s", window - total, "<");
-    print_chars(out, text + pos - before, before);
-    fprintf(out, "|=|");
-    total = print_chars(out, text + pos, window);
-    fprintf(out, "%*s\n", window - total, ">");
+    char *format = format_pos(text, pos);
+
+    if (format != NULL) {
+        fputs(format, out);
+        FREE(format);
+    }
+}
+
+int init_memstream(struct memstream *ms) {
+    MEMZERO(ms, 1);
+#if HAVE_OPEN_MEMSTREAM
+    ms->stream = open_memstream(&(ms->buf), &(ms->size));
+    return ms->stream == NULL ? -1 : 0;
+#else
+    ms->stream = tmpfile();
+    if (ms->stream == NULL) {
+        return -1;
+    }
+    return 0;
+#endif
+}
+
+int close_memstream(struct memstream *ms) {
+#if !HAVE_OPEN_MEMSTREAM
+    rewind(ms->stream);
+    ms->buf = fread_file_lim(ms->stream, MAX_READ_LEN, &(ms->size));
+#endif
+    if (fclose(ms->stream) == EOF) {
+        FREE(ms->buf);
+        ms->size = 0;
+        return -1;
+    }
+    return 0;
 }
 
 
