@@ -26,6 +26,7 @@
 #define DEBUG
 
 #include "list.h"
+#include "datadir.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -35,6 +36,44 @@
 #include <errno.h>
 #include <assert.h>
 
+/*
+ * Various parameters about env vars, special tree nodes etc.
+ */
+
+/* The default location for lens definitions */
+#define AUGEAS_LENS_DIR DATADIR "/augeas/lenses"
+
+/* The env var that points to the chroot holding files we may modify.
+   Mostly useful for testing */
+#define AUGEAS_ROOT_ENV "AUGEAS_ROOT"
+
+/* The root for actual file contents */
+#define AUGEAS_FILES_TREE "/files"
+
+/* Augeas reports some information in this subtree */
+#define AUGEAS_META_TREE "/augeas"
+
+/* Information about files */
+#define AUGEAS_META_FILES AUGEAS_META_TREE AUGEAS_FILES_TREE
+
+/* The root directory */
+#define AUGEAS_META_ROOT AUGEAS_META_TREE "/root"
+
+/* How we save files. One of 'backup', 'overwrite' or 'newfile' */
+#define AUGEAS_META_SAVE_MODE AUGEAS_META_TREE "/save"
+
+/* Name of env var that contains list of paths to search for additional
+   spec files */
+#define AUGEAS_LENS_ENV "AUGEAS_LENS_LIB"
+
+/* Fairly arbitrary bound on the length of the path we
+   accept from AUGEAS_SPEC_ENV */
+#define MAX_ENV_SIZE 4096
+
+/* Character separating paths in a list of paths */
+#define PATH_SEP_CHAR ':'
+
+
 #ifdef __GNUC__
 
 #ifndef __GNUC_PREREQ
@@ -195,27 +234,55 @@ struct augeas {
  */
 struct tree {
     struct tree *next;
-    const char  *label;      /* Last component of PATH */
+    char        *label;      /* Last component of PATH */
     struct tree *children;   /* List of children through NEXT */
-    const char  *value;
+    char        *value;
     int          dirty;
 };
 
 /* Allocate a new tree node with the given LABEL, VALUE, and CHILDREN,
  * which are not copied. The new tree is marked as dirty
  */
-struct tree *make_tree(const char *label, const char *value,
-                       struct tree *children);
+struct tree *make_tree(char *label, char *value, struct tree *children);
 
 int aug_tree_replace(struct augeas *aug, const char *path, struct tree *sub);
 
 int tree_rm(struct tree **tree, const char *path);
 struct tree *tree_set(struct tree *tree, const char *path, const char *value);
+int tree_insert(struct tree **tree, const char *path, const char *label,
+                int before);
 int free_tree(struct tree *tree);
 int print_tree(const struct tree *tree, FILE *out, const char *path,
                int pr_hidden);
 int tree_equal(const struct tree *t1, const struct tree *t2);
 
+/* Wrappers to simulate OPEN_MEMSTREAM where that's not available. The
+ * STREAM member is opened by INIT_MEMSTREAM and closed by
+ * CLOSE_MEMSTREAM. The BUF is allocated automatically, but can not be used
+ * until after CLOSE_MEMSTREAM has been called. It is the callers
+ * responsibility to free up BUF.
+ */
+struct memstream {
+    FILE   *stream;
+    char   *buf;
+    size_t size;
+};
+
+/* Initialize a memstream. On systems that have OPEN_MEMSTREAM, it is used
+ * to open MS->STREAM. On systems without OPEN_MEMSTREAM, MS->STREAM is
+ * backed by a temporary file.
+ *
+ * MS must be allocated in advance; INIT_MEMSTREAM initializes it.
+ */
+int init_memstream(struct memstream *ms);
+
+/* Close a memstream. After calling this, MS->STREAM can not be used
+ * anymore and a string representing whatever was written to it is
+ * available in MS->BUF. The caller must free MS->BUF.
+ *
+ * The caller must free the MEMSTREAM structure.
+ */
+int close_memstream(struct memstream *ms);
 #endif
 
 
