@@ -20,9 +20,10 @@
  * Author: David Lutterkort <dlutter@redhat.com>
  */
 
-#include "lens.h"
+#include <config.h>
 
-static struct regexp *regexp_digits = NULL;
+#include "lens.h"
+#include "memory.h"
 
 static struct value * typecheck_union(struct info *,
                                       struct lens *l1, struct lens *l2);
@@ -435,10 +436,21 @@ static struct regexp *make_key_regexp(struct info *info, const char *pat) {
     make_ref(regexp->pattern);
     regexp->info = ref(info);
     CALLOC(regexp->pattern->str, len);
-    snprintf((char *) regexp->pattern->str, len, "(%s)/", pat);
+    snprintf(regexp->pattern->str, len, "(%s)/", pat);
     return regexp;
 }
 
+static struct regexp *make_regexp_from_string(struct info *info,
+                                              struct string *string) {
+    struct regexp *r;
+    make_ref(r);
+    if (r != NULL) {
+        r->info = ref(info);
+        r->pattern = ref(string);
+    }
+    return r;
+}
+
 /* Calculate the regexp that matches the labels if the trees that L can
    generate.
 
@@ -449,35 +461,38 @@ static struct regexp *make_key_regexp(struct info *info, const char *pat) {
  */
 static struct regexp *lns_key_regexp(struct lens *l, struct value **exn) {
     static const struct string leaf_key_string = {
-        .ref = UINT_MAX, .str = "/"
+        .ref = REF_MAX, .str = (char *) "/"
     };
     static const struct string *const leaf_key_pat = &leaf_key_string;
 
+    static const struct string digits_string = {
+        .ref = REF_MAX, .str = (char *) "[0-9]+/"
+    };
+    static const struct string *const digits_pat = &digits_string;
+
     *exn = NULL;
     switch(l->tag) {
     case L_STORE:
-        {
-            struct regexp *r;
-            make_ref(r);
-            r->info = ref(l->info);
-            r->pattern = (struct string *) leaf_key_pat;
-            return r;
-        }
+        return make_regexp_from_string(l->info, (struct string *) leaf_key_pat);
     case L_DEL:
     case L_COUNTER:
         return NULL;
     case L_SEQ:
-        if (regexp_digits == NULL) {
-            regexp_digits = make_regexp(l->info, strdup("[0-9]+/"));
-            return regexp_digits;
-        } else {
-            return ref(regexp_digits);
-        }
-        break;
+        return make_regexp_from_string(l->info, (struct string *) digits_pat);
     case L_KEY:
         return make_key_regexp(l->info, l->regexp->pattern->str);
     case L_LABEL:
-        return make_key_regexp(l->info, l->string->str);
+        {
+            struct regexp *r = make_regexp_literal(l->info, l->string->str);
+            if (r == NULL)
+                return NULL;
+            if (REALLOC_N(r->pattern->str, strlen(r->pattern->str) + 2) == -1) {
+                unref(r, regexp);
+                return NULL;
+            }
+            strcat(r->pattern->str, "/");
+            return r;
+        }
     case L_CONCAT:
         {
             struct regexp *k = NULL;
