@@ -119,11 +119,22 @@
 #line 1 "parser.y"
 
 
+#include <config.h>
+
 #include "internal.h"
 #include "syntax.h"
 #include "list.h"
 #include <stdio.h>
 
+/* Work around a problem on FreeBSD where Bison looks for _STDLIB_H
+ * to see if stdlib.h has been included, but the system includes
+ * use _STDLIB_H_
+ */
+#if HAVE_STDLIB_H && ! defined _STDLIB_H
+#  include <stdlib.h>
+#  define _STDLIB_H 1
+#endif
+
 #define YYDEBUG 1
 
 int augl_parse_file(const char *name, struct term **term);
@@ -171,7 +182,7 @@ typedef struct info YYLTYPE;
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 60 "parser.y"
+#line 71 "parser.y"
 {
   struct term     *term;
   struct type    *type;
@@ -182,7 +193,7 @@ typedef union YYSTYPE
   enum quant_tag quant;
 }
 /* Line 187 of yacc.c.  */
-#line 186 "parser.c"
+#line 197 "parser.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -204,7 +215,7 @@ typedef struct YYLTYPE
 
 
 /* Copy the second part of user declarations.  */
-#line 81 "parser.y"
+#line 92 "parser.y"
 
 /* Lexer */
 extern int augl_lex (YYSTYPE * yylval_param,struct info * yylloc_param ,yyscan_t yyscanner);
@@ -220,14 +231,14 @@ static void augl_error(struct info *locp, struct term **term,
 
 /* TERM construction */
  static struct info *clone_info(struct info *locp);
- static struct term *make_module(const char *ident, const char *autoload,
+ static struct term *make_module(char *ident, char *autoload,
                                  struct term *decls,
                                  struct info *locp);
 
- static struct term *make_bind(const char *ident, struct term *params,
+ static struct term *make_bind(char *ident, struct term *params,
                              struct term *exp, struct term *decls,
                              struct info *locp);
- static struct term *make_let(const char *ident, struct term *params,
+ static struct term *make_let(char *ident, struct term *params,
                               struct term *exp, struct term *body,
                               struct info *locp);
  static struct term *make_binop(enum term_tag tag,
@@ -235,9 +246,9 @@ static void augl_error(struct info *locp, struct term **term,
                                struct info *locp);
  static struct term *make_unop(enum term_tag tag,
                               struct term *exp, struct info *locp);
- static struct term *make_ident(const char *qname, struct info *locp);
- static struct term *make_value_term(enum value_tag tag, const char *value,
-                               struct info *locp);
+ static struct term *make_ident(char *qname, struct info *locp);
+ static struct term *make_value_term(enum value_tag tag, char *value,
+                                     struct info *locp);
  static struct term *make_rep(struct term *exp, enum quant_tag quant,
                              struct info *locp);
 
@@ -263,7 +274,7 @@ static void augl_error(struct info *locp, struct term **term,
 
 
 /* Line 216 of yacc.c.  */
-#line 267 "parser.c"
+#line 278 "parser.c"
 
 #ifdef short
 # undef short
@@ -569,12 +580,12 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   140,   140,   143,   146,   148,   153,   158,   164,   168,
-     170,   173,   175,   179,   184,   186,   188,   191,   193,   195,
-     198,   200,   203,   205,   208,   210,   213,   215,   217,   219,
-     221,   224,   226,   229,   231,   233,   236,   238,   240,   242,
-     245,   248,   250,   253,   255,   257,   260,   262,   265,   267,
-     269,   271,   274,   279,   281,   285,   289,   291
+       0,   151,   151,   154,   157,   159,   164,   169,   175,   179,
+     181,   184,   186,   190,   195,   197,   199,   202,   204,   206,
+     209,   211,   214,   216,   219,   221,   224,   226,   228,   230,
+     232,   235,   237,   240,   242,   244,   247,   249,   251,   253,
+     256,   259,   261,   264,   266,   268,   271,   273,   276,   278,
+     280,   282,   285,   290,   292,   296,   300,   302
 };
 #endif
 
@@ -1582,22 +1593,22 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 141 "parser.y"
+#line 152 "parser.y"
     { (*term) = make_module((yyvsp[(2) - (5)].string), (yyvsp[(4) - (5)].string), (yyvsp[(5) - (5)].term), &(yylsp[(1) - (5)])); }
     break;
 
   case 3:
-#line 144 "parser.y"
+#line 155 "parser.y"
     { (yyval.string) = (yyvsp[(2) - (2)].string); }
     break;
 
   case 4:
-#line 146 "parser.y"
+#line 157 "parser.y"
     { (yyval.string) = NULL; }
     break;
 
   case 5:
-#line 149 "parser.y"
+#line 160 "parser.y"
     {
          LOC_MERGE((yylsp[(1) - (6)]), (yylsp[(1) - (6)]), (yylsp[(5) - (6)]));
          (yyval.term) = make_bind((yyvsp[(2) - (6)].string), (yyvsp[(3) - (6)].term), (yyvsp[(5) - (6)].term), (yyvsp[(6) - (6)].term), &(yylsp[(1) - (6)]));
@@ -1605,7 +1616,7 @@ yyreduce:
     break;
 
   case 6:
-#line 154 "parser.y"
+#line 165 "parser.y"
     {
          LOC_MERGE((yylsp[(1) - (5)]), (yylsp[(1) - (5)]), (yylsp[(4) - (5)]));
          (yyval.term) = make_test((yyvsp[(2) - (5)].term), (yyvsp[(4) - (5)].term), TR_CHECK, (yyvsp[(5) - (5)].term), &(yylsp[(1) - (5)]));
@@ -1613,7 +1624,7 @@ yyreduce:
     break;
 
   case 7:
-#line 159 "parser.y"
+#line 170 "parser.y"
     {
          LOC_MERGE((yylsp[(1) - (5)]), (yylsp[(1) - (5)]), (yylsp[(4) - (5)]));
          (yyval.term) = make_test((yyvsp[(2) - (5)].term), NULL, (yyvsp[(4) - (5)].intval), (yyvsp[(5) - (5)].term), &(yylsp[(1) - (5)]));
@@ -1621,32 +1632,32 @@ yyreduce:
     break;
 
   case 8:
-#line 164 "parser.y"
+#line 175 "parser.y"
     { (yyval.term) = NULL; }
     break;
 
   case 9:
-#line 169 "parser.y"
+#line 180 "parser.y"
     { (yyval.term) = make_get_test((yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 10:
-#line 171 "parser.y"
+#line 182 "parser.y"
     { (yyval.term) = make_put_test((yyvsp[(1) - (5)].term), (yyvsp[(3) - (5)].term), (yyvsp[(5) - (5)].term), &(yyloc)); }
     break;
 
   case 11:
-#line 174 "parser.y"
+#line 185 "parser.y"
     { (yyval.intval) = TR_PRINT; }
     break;
 
   case 12:
-#line 176 "parser.y"
+#line 187 "parser.y"
     { (yyval.intval) = TR_EXN; }
     break;
 
   case 13:
-#line 180 "parser.y"
+#line 191 "parser.y"
     {
        LOC_MERGE((yylsp[(1) - (7)]), (yylsp[(1) - (7)]), (yylsp[(6) - (7)]));
        (yyval.term) = make_let((yyvsp[(2) - (7)].string), (yyvsp[(3) - (7)].term), (yyvsp[(5) - (7)].term), (yyvsp[(7) - (7)].term), &(yylsp[(1) - (7)]));
@@ -1654,224 +1665,224 @@ yyreduce:
     break;
 
   case 15:
-#line 187 "parser.y"
+#line 198 "parser.y"
     { (yyval.term) = make_binop(A_COMPOSE, (yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 16:
-#line 189 "parser.y"
+#line 200 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 17:
-#line 192 "parser.y"
+#line 203 "parser.y"
     { (yyval.term) = make_binop(A_UNION, (yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 18:
-#line 194 "parser.y"
+#line 205 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 19:
-#line 196 "parser.y"
+#line 207 "parser.y"
     { (yyval.term) = make_tree_value((yyvsp[(1) - (1)].tree), &(yylsp[(1) - (1)])); }
     break;
 
   case 20:
-#line 199 "parser.y"
+#line 210 "parser.y"
     { (yyval.term) = make_binop(A_MINUS, (yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 21:
-#line 201 "parser.y"
+#line 212 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 22:
-#line 204 "parser.y"
+#line 215 "parser.y"
     { (yyval.term) = make_binop(A_CONCAT, (yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 23:
-#line 206 "parser.y"
+#line 217 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 24:
-#line 209 "parser.y"
+#line 220 "parser.y"
     { (yyval.term) = make_binop(A_APP, (yyvsp[(1) - (2)].term), (yyvsp[(2) - (2)].term), &(yyloc)); }
     break;
 
   case 25:
-#line 211 "parser.y"
+#line 222 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 26:
-#line 214 "parser.y"
+#line 225 "parser.y"
     { (yyval.term) = make_ident((yyvsp[(1) - (1)].string), &(yylsp[(1) - (1)])); }
     break;
 
   case 27:
-#line 216 "parser.y"
+#line 227 "parser.y"
     { (yyval.term) = make_value_term(V_STRING, (yyvsp[(1) - (1)].string), &(yylsp[(1) - (1)])); }
     break;
 
   case 28:
-#line 218 "parser.y"
+#line 229 "parser.y"
     { (yyval.term) = make_value_term(V_REGEXP, (yyvsp[(1) - (1)].string), &(yylsp[(1) - (1)])); }
     break;
 
   case 29:
-#line 220 "parser.y"
+#line 231 "parser.y"
     { (yyval.term) = (yyvsp[(2) - (3)].term); }
     break;
 
   case 30:
-#line 222 "parser.y"
+#line 233 "parser.y"
     { (yyval.term) = make_unop(A_BRACKET, (yyvsp[(2) - (3)].term), &(yyloc)); }
     break;
 
   case 31:
-#line 225 "parser.y"
+#line 236 "parser.y"
     { (yyval.term) = make_rep((yyvsp[(1) - (2)].term), (yyvsp[(2) - (2)].quant), &(yyloc)); }
     break;
 
   case 32:
-#line 227 "parser.y"
+#line 238 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 33:
-#line 230 "parser.y"
+#line 241 "parser.y"
     { (yyval.quant) = Q_STAR; }
     break;
 
   case 34:
-#line 232 "parser.y"
+#line 243 "parser.y"
     { (yyval.quant) = Q_PLUS; }
     break;
 
   case 35:
-#line 234 "parser.y"
+#line 245 "parser.y"
     { (yyval.quant) = Q_MAYBE; }
     break;
 
   case 36:
-#line 237 "parser.y"
+#line 248 "parser.y"
     { (yyval.string) = (yyvsp[(1) - (1)].string); }
     break;
 
   case 37:
-#line 239 "parser.y"
+#line 250 "parser.y"
     { (yyval.string) = (yyvsp[(1) - (1)].string); }
     break;
 
   case 38:
-#line 241 "parser.y"
+#line 252 "parser.y"
     { (yyval.string) = strdup("get"); }
     break;
 
   case 39:
-#line 243 "parser.y"
+#line 254 "parser.y"
     { (yyval.string) = strdup("put"); }
     break;
 
   case 40:
-#line 246 "parser.y"
+#line 257 "parser.y"
     { (yyval.term) = (yyvsp[(2) - (2)].term); list_cons((yyval.term), (yyvsp[(1) - (2)].term)); }
     break;
 
   case 41:
-#line 248 "parser.y"
+#line 259 "parser.y"
     { (yyval.term) = NULL; }
     break;
 
   case 42:
-#line 251 "parser.y"
+#line 262 "parser.y"
     { (yyval.term) = make_param((yyvsp[(2) - (5)].string), (yyvsp[(4) - (5)].type), clone_info(&(yylsp[(1) - (5)]))); }
     break;
 
   case 43:
-#line 254 "parser.y"
+#line 265 "parser.y"
     { (yyval.string) = (yyvsp[(1) - (1)].string); }
     break;
 
   case 44:
-#line 256 "parser.y"
+#line 267 "parser.y"
     { (yyval.string) = strdup("get"); }
     break;
 
   case 45:
-#line 258 "parser.y"
+#line 269 "parser.y"
     { (yyval.string) = strdup("put"); }
     break;
 
   case 46:
-#line 261 "parser.y"
+#line 272 "parser.y"
     { (yyval.type) = make_arrow_type((yyvsp[(1) - (3)].type), (yyvsp[(3) - (3)].type)); }
     break;
 
   case 47:
-#line 263 "parser.y"
+#line 274 "parser.y"
     { (yyval.type) = (yyvsp[(1) - (1)].type); }
     break;
 
   case 48:
-#line 266 "parser.y"
+#line 277 "parser.y"
     { (yyval.type) = make_base_type(T_STRING); }
     break;
 
   case 49:
-#line 268 "parser.y"
+#line 279 "parser.y"
     { (yyval.type) = make_base_type(T_REGEXP); }
     break;
 
   case 50:
-#line 270 "parser.y"
+#line 281 "parser.y"
     { (yyval.type) = make_base_type(T_LENS); }
     break;
 
   case 51:
-#line 272 "parser.y"
+#line 283 "parser.y"
     { (yyval.type) = (yyvsp[(2) - (3)].type); }
     break;
 
   case 52:
-#line 275 "parser.y"
+#line 286 "parser.y"
     {
               (yyval.tree) = tree_concat((yyvsp[(2) - (4)].tree), (yyvsp[(4) - (4)].tree));
             }
     break;
 
   case 53:
-#line 279 "parser.y"
+#line 290 "parser.y"
     { (yyval.tree) = NULL; }
     break;
 
   case 54:
-#line 282 "parser.y"
+#line 293 "parser.y"
     {
                (yyval.tree) = make_tree((yyvsp[(1) - (2)].string), NULL, (yyvsp[(2) - (2)].tree));
              }
     break;
 
   case 55:
-#line 286 "parser.y"
+#line 297 "parser.y"
     {
                (yyval.tree) = make_tree((yyvsp[(1) - (4)].string), (yyvsp[(3) - (4)].string), (yyvsp[(4) - (4)].tree));
              }
     break;
 
   case 57:
-#line 291 "parser.y"
+#line 302 "parser.y"
     { (yyval.string) = NULL; }
     break;
 
 
 /* Line 1267 of yacc.c.  */
-#line 1875 "parser.c"
+#line 1886 "parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2091,7 +2102,7 @@ yyreturn:
 }
 
 
-#line 292 "parser.y"
+#line 303 "parser.y"
 
 
 int augl_parse_file(const char *name, struct term **term) {
@@ -2148,7 +2159,7 @@ static struct term *make_term_locp(enum term_tag tag, struct info *locp) {
   return make_term(tag, info);
 }
 
-static struct term *make_module(const char *ident, const char *autoload,
+static struct term *make_module(char *ident, char *autoload,
                                 struct term *decls,
                                 struct info *locp) {
   struct term *term = make_term_locp(A_MODULE, locp);
@@ -2158,7 +2169,7 @@ static struct term *make_module(const char *ident, const char *autoload,
   return term;
 }
 
-static struct term *make_bind(const char *ident, struct term *params,
+static struct term *make_bind(char *ident, struct term *params,
                               struct term *exp, struct term *decls,
                               struct info *locp) {
   struct term *term = make_term_locp(A_BIND, locp);
@@ -2171,7 +2182,7 @@ static struct term *make_bind(const char *ident, struct term *params,
   return decls;
 }
 
-static struct term *make_let(const char *ident, struct term *params,
+static struct term *make_let(char *ident, struct term *params,
                              struct term *exp, struct term *body,
                              struct info *locp) {
   /* let f (x:string) = "f " . x in
@@ -2208,13 +2219,13 @@ static struct term *make_unop(enum term_tag tag, struct term *exp,
   return term;
 }
 
-static struct term *make_ident(const char *qname, struct info *locp) {
+static struct term *make_ident(char *qname, struct info *locp) {
   struct term *term = make_term_locp(A_IDENT, locp);
   term->ident = make_string(qname);
   return term;
 }
 
-static struct term *make_value_term(enum value_tag tag, const char *value,
+static struct term *make_value_term(enum value_tag tag, char *value,
                                     struct info *locp) {
   assert(tag == V_STRING || tag == V_REGEXP);
   struct term *term = make_term_locp(A_VALUE, locp);
