@@ -20,6 +20,8 @@
  * Author: David Lutterkort <dlutter@redhat.com>
  */
 
+#include <config.h>
+
 #include <stdarg.h>
 #include "syntax.h"
 
@@ -43,7 +45,7 @@ struct split {
     struct split *next;
     struct tree  *tree;
     struct tree  *follow;
-    const char   *labels;
+    char         *labels;
     size_t        start;
     size_t        end;
 };
@@ -110,14 +112,15 @@ static struct split *make_split(struct tree *tree) {
             split->end += strlen(t->label);
         split->end += 1;
     }
-    char *l;
-    CALLOC(l, split->end + 1);
+    char *labels;
+    CALLOC(labels, split->end + 1);
+    char *l = labels;
     for (struct tree *t = tree; t != NULL; t = t->next) {
         if (t->label != NULL)
-            strcat(l, t->label);
-        strcat(l, "/");
+            l = stpcpy(l, t->label);
+        l = stpcpy(l, "/");
     }
-    split->labels = l;
+    split->labels = labels;
     return split;
 }
 
@@ -125,13 +128,13 @@ static void free_split(struct split *split) {
     if (split == NULL)
         return;
 
-    free((char *) split->labels);
+    free(split->labels);
     free(split);
 }
 
 static void split_append(struct split **split,
                          struct tree *tree, struct tree *follow,
-                         const char *labels, size_t start, size_t end) {
+                         char *labels, size_t start, size_t end) {
     struct split *sp;
     CALLOC(sp, 1);
     sp->tree = tree;
@@ -502,8 +505,11 @@ static void put_quant_maybe(struct lens *lens, struct state *state) {
 }
 
 static void put_store(struct lens *lens, struct state *state) {
-    if (regexp_match(lens->regexp, state->value, strlen(state->value),
-                     0, NULL) != strlen(state->value)) {
+    if (state->value == NULL) {
+        put_error(state, lens,
+                  "Can not store a nonexistent (NULL) value");
+    } else if (regexp_match(lens->regexp, state->value, strlen(state->value),
+                            0, NULL) != strlen(state->value)) {
         put_error(state, lens,
                   "Value '%s' does not match regexp /%s/ in store lens",
                   state->value, lens->regexp->pattern->str);
