@@ -20,11 +20,13 @@
  * Author: David Lutterkort <dlutter@redhat.com>
  */
 
+#include <config.h>
+
 #include "syntax.h"
 #include "memory.h"
 
 static const struct string empty_pattern_string = {
-    .ref = REF_MAX, .str = "()"
+    .ref = REF_MAX, .str = (char *) "()"
 };
 
 static const struct string *const empty_pattern = &empty_pattern_string;
@@ -43,7 +45,7 @@ void print_regexp(FILE *out, struct regexp *r) {
     fputc('/', out);
 }
 
-struct regexp *make_regexp(struct info *info, const char *pat) {
+struct regexp *make_regexp(struct info *info, char *pat) {
     struct regexp *regexp;
 
     make_ref(regexp);
@@ -75,10 +77,10 @@ struct regexp *make_regexp_literal(struct info *info, const char *text) {
     CALLOC(pattern, 2*strlen(text)+1);
     p = pattern;
     for (const char *t = text; *t != '\0'; t++) {
-        if (*t == '\\') {
+        if ((*t == '\\') && t[1]) {
             *p++ = *t++;
             *p++ = *t;
-        } else if (strchr(".{}[]()+*?", *t) != NULL) {
+        } else if (strchr(".|{}[]()+*?", *t) != NULL) {
             *p++ = '\\';
             *p++ = *t;
         } else {
