@@ -2,7 +2,7 @@
 entry_last = "/files/etc/pam.d/newrole/10000"
 commands="
 ins entry after #{entry_last}
-set #{entry_last}/type test
+set #{entry_last}/type auth
 set #{entry_last}/control include
 set #{entry_last}/module system-auth
 save
@@ -15,5 +15,5 @@ diff["/etc/pam.d/newrole"] = <<TXT
  account    include\tsystem-auth
  password   include\tsystem-auth
  session    required\tpam_namespace.so unmnt_remnt no_unmount_on_close
-+test\tinclude\tsystem-auth
++auth\tinclude\tsystem-auth
 TXT
